
#region DEMO: Query parameters and advanced queries

#region -ExpandProperty parameter

Get-MgGroup -GroupId '55d64e2b-f6a1-4147-8520-5e3347063d6d' | fl
Get-MgGroup -GroupId '55d64e2b-f6a1-4147-8520-5e3347063d6d' -Expand members | fl

Get-MgGroup -GroupId '55d64e2b-f6a1-4147-8520-5e3347063d6d' -Expand members | 
  Select -ExpandProperty members

Get-MgGroup -GroupId '55d64e2b-f6a1-4147-8520-5e3347063d6d' -Expand members | 
  Select -ExpandProperty members | fl

Get-MgGroup -GroupId '55d64e2b-f6a1-4147-8520-5e3347063d6d' -Expand members -Select DisplayName |
  Select -Property DisplayName -ExpandProperty members | fl

Get-MgGroup -GroupId '55d64e2b-f6a1-4147-8520-5e3347063d6d' -Expand members | 
  Select -ExpandProperty members |
  Select -ExpandProperty Id

#endregion

#region -Search parameter; pay attention to the quotes

Get-MgUser -Search '"DisplayName:Al"'
Get-MgUser -Search '"DisplayName:Al"' -ConsistencyLevel eventual -Count UserCount

Get-MgUser -Search "'DisplayName:Al'" -ConsistencyLevel eventual -Count UserCount
Get-MgUser -Search "`'DisplayName:Al`'" -ConsistencyLevel eventual -Count UserCount
Get-MgUser -Search `"'DisplayName:Al'`" -ConsistencyLevel eventual -Count UserCount

Get-MgUser -Search ""DisplayName:Al"" -ConsistencyLevel eventual -Count UserCount
Get-MgUser -Search "`"DisplayName:Al`"" -ConsistencyLevel eventual -Count UserCount
Get-MgUser -Search `""DisplayName:Al"`" -ConsistencyLevel eventual -Count UserCount

Get-MgUser -Search ''DisplayName:Al'' -ConsistencyLevel eventual -Count UserCount
Get-MgUser -Search '`'DisplayName:Al`'' -ConsistencyLevel eventual -Count UserCount
Get-MgUser -Search `''DisplayName:Al'`' -ConsistencyLevel eventual -Count UserCount

#endregion

#region Examples using the filter query operator

# Get all users with the name Ale across multiple properties.	
Get-MgUser -Filter "startswith(displayName,'ale') or startswith(givenName,'ale') or startswith(surname,'ale') or startswith(mail,'ale') or startswith(userPrincipalName,'ale')"

# Get all the signed-in user's events that start after 7/1/2017.	
# NOTE: The dateTime property is a String type.
Get-MgUserEvent -UserId 'aleksandar@DOMAINNAME.onmicrosoft.com' -Filter "start/dateTime ge '2017-07-01T08:00'"

# Get all emails from a specific address received by the signed-in user.	
Get-MgUserMessage -UserId 'aleksandar@DOMAINNAME.onmicrosoft.com' -Filter "from/emailAddress/address eq 'azure-noreply@microsoft.com'"

# Get all unread mail in the signed-in user's Inbox.	
Get-MgUserMessage -UserId 'aleksandar@DOMAINNAME.onmicrosoft.com' -Filter "isRead eq false"

# Get all users in the Retail and Sales departments.	
Get-MgUser -Filter "department in ('Retail', 'Sales')"

#region ADVANCED QUERIES

# Get all users with mail domain equal to 'hotmail.com'	
GET ~/users?$count=true&$filter=endsWith(mail,'@hotmail.com')*
Get-MgUser -Filter "endsWith(mail,'@live.com')" -CountVariable CountVar -ConsistencyLevel eventual

# Get all users without assigned licenses	
GET ~/users?$filter=assignedLicenses/$count eq 0&$count=true*
Get-MgUser -Filter "assignedLicenses/`$count eq 0" -CountVariable CountVar -ConsistencyLevel eventual

# List users with a particular service plan that is in a suspended state.	
GET ~/users?$filter=assignedPlans/any(a:a/servicePlanId eq 2e2ddb96-6af9-4b1d-a3f0-d6ecfd22edb2 and a/capabilityStatus eq 'Suspended')&$count=true*

# List all non-Microsoft 365 groups in an organization.	
GET ~/groups?$filter=NOT groupTypes/any(c:c eq 'Unified')&$count=true*
Get-MgGroup -Filter "NOT groupTypes/any(c:c eq 'Unified')" -CountVariable CountVar -ConsistencyLevel eventual

# List all users whose company name isn't undefined (that is, not a null value) or Microsoft.	
GET ~/users?$filter=companyName ne null and NOT(companyName eq 'Microsoft')&$count=true*
Get-MgUser -Filter "companyName ne null and NOT(companyName eq 'Microsoft')" -CountVariable CountVar -ConsistencyLevel eventual

# List all users whose company name is either undefined or Microsoft.	
GET ~/users?$filter=companyName in (null, 'Microsoft')&$count=true*
Get-MgUser -Filter "companyName in (null, 'Microsoft')" -CountVariable CountVar -ConsistencyLevel eventual

# Use OData cast to get transitive membership in groups with a display name that starts with 'a' including a count of returned objects.	
GET ~/me/transitiveMemberOf/microsoft.graph.group?$count=true&$filter=startswith(displayName, 'a')*
https://graph.microsoft.com/v1.0/me/transitiveMemberOf/microsoft.graph.group?$count=true&$filter=startswith(displayName, 't') # works in GE
find-MgGraphCommand -uri '/users/{id}/transitivememberof'
Get-MgUserTransitiveMemberOf -UserId 'aleksandar@DOMAINNAME.onmicrosoft.com' -Filter "startswith(displayName, 't')" -CountVariable CountVar -ConsistencyLevel eventual | fl

#endregion

#endregion

$userId = 'aleksandar@DOMAINNAME.onmicrosoft.com'
Get-MgUserMessage -UserId $userId -Search '"pim"' 

Get-MgUserMessage -UserId $userId -Search '"hasAttachments:true"'

# display name of the user.	
Get-MgUser -Search '"displayName:Bowen"' -ConsistencyLevel eventual

# display name or mail of the user.	
Get-MgUser -Search '"displayName:Diego" OR "mail:Megan"' -ConsistencyLevel eventual

# display name or description of the group.	
Get-MgGroup -Search '"description:Visual Studio MSDN" AND ("displayName:admins" OR "displayName:administrators")' -ConsistencyLevel eventual

#region Advanced queries

# To retrieve only inactive user accounts, you can run either of these queries that use the $filter query parameter.

# Option 1: Use the $filter query parameter with the eq operator. This request will work by default.

Get-MgUser -Filter "accountEnabled eq false" 

# Option 2: Use the $filter query parameter with the ne operator. This request is not supported by default because the ne operator is only supported in advanced queries.

Get-MgUser -Filter "accountEnabled ne true" -CountVariable CountVar -ConsistencyLevel eventual

#endregion

#region Error handling for advanced queries on directory objects

# For directory objects, $search works only in advanced queries.
# If the ConsistencyLevel header is not specified, the request returns an error.
Get-MgApplication -Search '"displayName:Browser"'
# Get-MgApplication_List: Request with $search query parameter only works through MSGraph with a special request header: 'ConsistencyLevel: eventual'
Get-MgApplication -Search '"displayName:Graph"' -Count count -ConsistencyLevel eventual

# If a property or query parameter in the URL is supported only in advanced queries but either the ConsistencyLevel header or the $count=true query string is missing, the request returns an error.
Get-MgUser -Filter "endsWith(mail,'@live.com')"
# Get-MgUser_List: Filter operator 'SuffixMatch' is not supported.
# "Unsupported Query."
Get-MgUser -Filter "endsWith(mail,'@live.com')" -CountVariable CountVar -ConsistencyLevel eventual

# If a property has not been indexed to support a query parameter, even if the advanced query parameters are specified, the request returns an error.
Get-MgGroup -Filter "createdDateTime ge 2021-11-01" -CountVariable CountVar -ConsistencyLevel eventual 
# Get-MgGroup_List: The type of the value doesn't match the property whose type is DateTime
# "The request uses a filter property that is not indexed"

# However, it is important to note that query parameters specified in a request might fail silently.
# This can be true for unsupported query parameters as well as for unsupported combinations of query parameters. 
# SILENTLY FAILS
Get-MgUser -CountVariable userCount
$userCount
# WORKS
Get-MgUser -CountVariable userCount -ConsistencyLevel eventual
$userCount
#endregion
