
#region DEMO: Working with groups

# To list only Microsoft 365 groups (aka unified groups), apply a filter on groupTypes
Get-MgGroup -Filter "groupTypes/any(c:c eq 'Unified')"

Get-MgGroup -Filter "startswith(displayName, 'sg')" -CountVariable CountVar -Top 3 -Sort "displayName" -ConsistencyLevel eventual 

$group = Get-MgGroup -Filter "displayName eq 'testgroup1'"

Get-MgGroupMember -GroupId $group.Id -ov members

Get-MgGroupMember -GroupId $group.Id -Filter "startswith(displayName, 'a')" -CountVariable CountVar -ConsistencyLevel eventual 

# Pipeline

# this fails
Get-MgGroup -Filter "displayName eq 'sg-test1'" | Update-MgGroup -Description "Test group 1"
# Update-MgGroup_UpdateViaIdentity$Expanded: The pipeline has been stopped.
# Exception: InputObject has null value for InputObject.GroupId

# this works
Get-MgGroup -Filter "displayName eq 'sg-test1'" | ForEach-Object { @{GroupId = $_.Id} } | Update-MgGroup -Description "Test group 1"

# TypeName: Microsoft.Graph.PowerShell.Models.MicrosoftGraphDirectoryObject
$members | Get-Member

$members | fl *

$members.ToJsonString() | ConvertFrom-Json | fl *

# Translate Directory Objects to Users 

Get-MgGroupOwner -GroupId $Group.Id
Get-MgGroupOwner -GroupId $Group.Id | gm # DirectoryObject
(Get-MgGroupOwner -GroupId $Group.Id).ToJsonString()
(Get-MgGroupOwner -GroupId $Group.Id).ToJsonString() | ConvertFrom-Json
(Get-MgGroupOwner -GroupId $Group.Id).AdditionalProperties # hash table
(Get-MgGroupOwner -GroupId $Group.Id).AdditionalProperties | ForEach-Object {New-Object PSObject -Property $_} # PSCustomObject 

Get-MgGroupOwner -GroupId $Group.Id | ForEach-Object { @{ UserId=$_.Id}} | Get-MgUser | gm # MicrosoftGraphUser
Get-MgGroupOwner -GroupId $Group.Id | ForEach-Object { @{ UserId=$_.Id}} | Get-MgUser | fl *

Get-MgGroupTransitiveMember -GroupId $group.Id -ov transitiveMembers

# filter only groups
$transitiveMembers.tojsonstring() | ConvertFrom-Json
$transitiveMembers.tojsonstring() | ConvertFrom-Json | where '@odata.type' -match 'group' | fl *
$transitiveMembers.AdditionalProperties| where {$_.'@odata.type' -match 'group'}

# Get groups and administrative units that the group is a direct member of
Get-MgGroupMemberOf -GroupId 07a075f5-22a5-4b09-839e-cf8960f697e4 -ov MemberOf
$MemberOf.tojsonstring() | ConvertFrom-Json

#region Different ways to create a group

<# Permissions
User Administrator, Groups Administrator or Global Administrator Azure AD Role(s)
Graph API Scopes:
Delegated: Group.ReadWrite.All, Directory.ReadWrite.All
Application: Group.Create, Group.ReadWrite.All, Directory.ReadWrite.All
#>

#region Create a Microsoft 365 group with no owner and no members

# Because the owners have not been specified, the calling user is automatically added as the owner of the group.

$params = @{
	description = "Microsoft 365 group"
	displayName = "M365G-NoOwnerNoMember"
	groupTypes = @(
		"Unified"
	)
	mailEnabled = $true
	mailNickname = "M365group"
	securityEnabled = $false
}

New-MgGroup -BodyParameter $params

#endregion

#region Create a security-enabled group with no owner and no members

$GroupParams = @{
    DisplayName = "SG-NoOwnerNoMember"
    GroupTypes = @(
    )
    SecurityEnabled     = $true
    IsAssignableToRole  = $false
    MailEnabled         = $false
    MailNickname        = (New-Guid).Guid.Substring(0,10)
}

New-MgGroup -BodyParameter $GroupParams

# Create a security-enabled group with no owner and no members
New-MgGroup -DisplayName "SG-SecurityNoOwnerNoMember" -GroupTypes @() -SecurityEnabled $true -IsAssignableToRole $false -MailEnabled $false -MailNickname (New-Guid).Guid.Substring(0,10)

#endregion

#region Create a security group with an owner

# a single owner
$GroupParams = @{
    DisplayName = "SG-SecurityOwner"
    GroupTypes = @(
    )
    SecurityEnabled     = $true
    IsAssignableToRole  = $false
    MailEnabled         = $false
    MailNickname        = (New-Guid).Guid.Substring(0,10)
    Owners = @(
        @{
            Id = "cc6166e6-acf1-419b-b06d-f44abb945bde"
        }
    )
}

New-MgGroup -BodyParameter $GroupParams

# multiple owners
$GroupParams = @{
    DisplayName = "SG-SecurityGroupWithOwner"
    GroupTypes = @(
    )
    SecurityEnabled     = $true
    IsAssignableToRole  = $false
    MailEnabled         = $false
    MailNickname        = (New-Guid).Guid.Substring(0,10)
    "Owners@odata.bind" = @(
        "https://graph.microsoft.com/v1.0/me",
        "https://graph.microsoft.com/v1.0/users/diegos@DOMAINNAME.onmicrosoft.com",
        "https://graph.microsoft.com/v1.0/users/647e9c5e-xxxx-xxxx-xxxx-xxxxxxxxxxxx"
        "https://graph.microsoft.com/v1.0/servicePrincipals/50ded543-xxxx-xxxx-xxxx-xxxxxxxxxxxx"
    )
}
New-MgGroup -BodyParameter $GroupParams

#endregion

#region Create a security group with members and owners

$GroupParam = @{
    DisplayName = "SG-SecurityGroupWithOwnerAndMembers"
    GroupTypes = @(
    )
    SecurityEnabled     = $true
    IsAssignableToRole  = $false
    MailEnabled         = $false
    MailNickname        = (New-Guid).Guid.Substring(0,10)
    "Owners@odata.bind" = @(
        "https://graph.microsoft.com/v1.0/me",
        "https://graph.microsoft.com/v1.0/users/diegos@DOMAINNAME.onmicrosoft.com"
    )
    "Members@odata.bind" = @(
        "https://graph.microsoft.com/v1.0/me",
        "https://graph.microsoft.com/v1.0/users/adelev@DOMAINNAME.onmicrosoft.com"
    )
}
New-MgGroup -BodyParameter $GroupParam

#endregion

#region Create dynamic groups

$GroupParam = @{
  DisplayName                   = 'SG-DynamicSecurityGroup'
  GroupTypes                    = @(
    'DynamicMembership'
  )
  SecurityEnabled               = $true
  IsAssignableToRole            = $false
  MailEnabled                   = $false
  membershipRuleProcessingState = 'On'
  MembershipRule                = '(user.accountEnabled -eq true) and (user.userPrincipalName -eq "diegos@DOMAINNAME.onmicrosoft.com")'
  MailNickname                  = (New-Guid).Guid.Substring(0, 10)
  'Owners@odata.bind'           = @(
    'https://graph.microsoft.com/v1.0/me'
  )
}

New-MgGroup -BodyParameter $GroupParam

New-MgGroup -DisplayName 'MyDynSecurityGroup' `
-MailEnabled:$False `
-MailNickName 'MyDynSecurityGroup' `
-SecurityEnabled `
-GroupTypes DynamicMembership `
-MembershipRule '(user.userPrincipalName -contains "DOMAINNAME.onmicrosoft.com") and (user.accountEnabled -eq True)' `
-MembershipRuleProcessingState On

New-MgGroup -DisplayName 'MyDynM365Group' `
-MailEnabled:$False `
-MailNickName 'MyDynM365Group' `
-SecurityEnabled `
-GroupTypes DynamicMembership,Unified `
-MembershipRule '(user.userPrincipalName -contains "DOMAINNAME.onmicrosoft.com")' `
-MembershipRuleProcessingState On

$GroupBody = @{
    DisplayName = "MyDynM365Group"
     MailEnabled         = $true
     MailNickname        = "MyDynM365Group"
     SecurityEnabled     = $true
     GroupTypes = @(
        'DynamicMembership','Unified'
     )
     MembershipRule =  '(user.userPrincipalName -contains "DiegoS")'
     membershipRuleProcessingState = 'On'
}

New-MgGroup -BodyParameter $GroupBody

# List all dynamic groups
Get-MgGroup -Filter "(groupTypes/any(c:c eq 'DynamicMembership'))" -CountVariable CountVar

POST https://graph.microsoft.com/beta/groups/evaluateDynamicMembership

$params = @{
	MemberId = "4b1fda38-cc2e-42e9-afd1-18f1d1887487" # user id
	MembershipRule = '(user.userPrincipalName -contains "DiegoS")' # outer single quotes
}

Test-MgBetaGroupDynamicMembershipRule -BodyParameter $params

#endregion

#endregion

#endregion
