#region DEMO: Using Microsoft Graph API with PowerShell

# Invoke-MgGraphRequest issues REST API requests to the Graph API. It works for any Graph API if you know the REST URI, method and optional body parameter. This command is especially useful for accessing APIs for which there isn't an equivalent cmdlet yet.

Get-Command Invoke-MgGraphRequest -Syntax

<#
Invoke-MgGraphRequest [[-Method] <GraphRequestMethod>] # DELETE GET PATCH POST PUT
[-Uri] <uri> 
[[-Body] <Object>] 
[[-Headers] <IDictionary>] 
[[-OutputFilePath] <string>] 
[-InferOutputFileName] 
[[-InputFilePath] <string>] 
[-PassThru] 
[-SkipHeaderValidation] 
[[-ContentType] <string>] 
[[-SessionVariable] <string>] 
[[-ResponseHeadersVariable] <string>] 
[[-StatusCodeVariable] <string>] 
[-SkipHttpErrorCheck] 
[[-GraphRequestSession] <GraphRequestSession>] 
[[-UserAgent] <string>] 
[[-OutputType] <OutputType>] # HashTable HttpResponseMessage Json PSObject
[-Break] [<CommonParameters>]
#>

# What is the REST URI for the Microsoft Graph API?

# The URI (or Universal Resource Identifier) identifies where a specific resource can be found in Microsoft Graph.

<#
# Microsoft Graph REST API v1.0 endpoint reference
https://docs.microsoft.com/en-us/graph/api/overview?view=graph-rest-1.0
# user resource type
https://docs.microsoft.com/en-us/graph/api/resources/user?view=graph-rest-1.0
# Get a user
https://docs.microsoft.com/en-us/graph/api/user-get?view=graph-rest-1.0&tabs=http

# Microsoft Graph REST API beta endpoint reference
https://learn.microsoft.com/en-us/graph/api/overview?view=graph-rest-beta

https://graph.microsoft.com/v1.0/users/{id}

https://graph.microsoft.com – endpoint of the Microsoft Graph service

v1.0/ – Microsoft Graph profile version

devices/ – a path to the resource

{id} – the unique resource ID

#>

# How to find your required REST URI?

# Use the Graph Explorer > Resources tab
# https://developer.microsoft.com/en-us/graph/graph-explorer
# https://aka.ms/ge

# Use the Graph Explorer > Sample Queries tab

# Use Graph X-Ray (a Chrome extension and an Edge add-on)
# https://graphxray.merill.net/

#region Examples

<#
GET /groups?$select=id,displayName,mailEnabled,securityEnabled,groupTypes,onPremisesSyncEnabled,mail,isAssignableToRole,writebackConfiguration,isManagementRestricted,resourceProvisioningOptions,expirationDateTime,createdDateTime,membershipRuleProcessingState&$top=100&$orderby=displayName asc&$count=true
#>
Import-Module Microsoft.Graph.Groups

Get-MgGroup -Property "id,displayName,mailEnabled,securityEnabled,groupTypes,onPremisesSyncEnabled,mail,isAssignableToRole,writebackConfiguration,isManagementRestricted,resourceProvisioningOptions,expirationDateTime,createdDateTime,membershipRuleProcessingState" -Top 100 -Sort "displayName asc" -CountVariable CountVar 

<#
GET https://graph.microsoft.com/beta/groups/XXXXXXXX-22a5-4b09-839e-cf8960f697e4?$select=id,deletedDateTime,createdDateTime,description,displayName,expirationDateTime,groupTypes,mail,mailEnabled,mailNickname,membershipRule,membershipRuleProcessingState,onPremisesLastSyncDateTime,onPremisesSecurityIdentifier,onPremisesSyncEnabled,proxyAddresses,renewedDateTime,resourceBehaviorOptions,resourceProvisioningOptions,securityEnabled,visibility,onPremisesProvisioningErrors,assignedLicenses,isAssignableToRole,isManagementRestricted,membershipRuleProcessingStatus,assignedLabels,writebackConfiguration
#>
Import-Module Microsoft.Graph.Groups

Get-MgGroup -GroupId $groupId -Property "id,deletedDateTime,createdDateTime,description,displayName,expirationDateTime,groupTypes,mail,mailEnabled,mailNickname,membershipRule,membershipRuleProcessingState,onPremisesLastSyncDateTime,onPremisesSecurityIdentifier,onPremisesSyncEnabled,proxyAddresses,renewedDateTime,resourceBehaviorOptions,resourceProvisioningOptions,securityEnabled,visibility,onPremisesProvisioningErrors,assignedLicenses,isAssignableToRole,isManagementRestricted,membershipRuleProcessingStatus,assignedLabels,writebackConfiguration" 

<#
GET /groups/XXXXXXXX-22a5-4b09-839e-cf8960f697e4/appRoleAssignments?$select=id
#>
Import-Module Microsoft.Graph.Applications

Get-MgGroupAppRoleAssignment -GroupId $groupId -Property "id" 

<#
GET https://graph.microsoft.com/beta/groups/XXXXXXXX-22a5-4b09-839e-cf8960f697e4?$select=id,deletedDateTime,createdDateTime,description,displayName,expirationDateTime,groupTypes,mail,mailEnabled,mailNickname,membershipRule,membershipRuleProcessingState,onPremisesLastSyncDateTime,onPremisesSecurityIdentifier,onPremisesSyncEnabled,proxyAddresses,renewedDateTime,resourceBehaviorOptions,resourceProvisioningOptions,securityEnabled,visibility,onPremisesProvisioningErrors,assignedLicenses,isAssignableToRole,isManagementRestricted,membershipRuleProcessingStatus,assignedLabels,writebackConfiguration
#>
Import-Module Microsoft.Graph.Groups

Get-MgGroup -GroupId $groupId -Property "id,deletedDateTime,createdDateTime,description,displayName,expirationDateTime,groupTypes,mail,mailEnabled,mailNickname,membershipRule,membershipRuleProcessingState,onPremisesLastSyncDateTime,onPremisesSecurityIdentifier,onPremisesSyncEnabled,proxyAddresses,renewedDateTime,resourceBehaviorOptions,resourceProvisioningOptions,securityEnabled,visibility,onPremisesProvisioningErrors,assignedLicenses,isAssignableToRole,isManagementRestricted,membershipRuleProcessingStatus,assignedLabels,writebackConfiguration" 

<#
GET https://graph.microsoft.com/beta/groups/XXXXXXXX-22a5-4b09-839e-cf8960f697e4/members?$select=id,displayName,userType,appId,mail,onPremisesSyncEnabled,deviceId&$top=20&$orderby=displayName%20asc&$count=true
#>
Import-Module Microsoft.Graph.Groups

Get-MgGroupMember -GroupId $groupId -Property "id,displayName,userType,appId,mail,onPremisesSyncEnabled,deviceId" -Top 20 -Sort "displayName%20asc" -CountVariable CountVar 

<#
GET https://graph.microsoft.com/beta/groups/XXXXXXXX-22a5-4b09-839e-cf8960f697e4/transitiveMembers?$select=id,displayName,userType,appId,mail,onPremisesSyncEnabled,deviceId&$top=20&$orderby=displayName%20asc&$count=true
#>
Import-Module Microsoft.Graph.Groups

Get-MgGroupTransitiveMember -GroupId $groupId -Property "id,displayName,userType,appId,mail,onPremisesSyncEnabled,deviceId" -Top 20 -Sort "displayName%20asc" -CountVariable CountVar 

# we need to tweak it a little and change "displayName%20asc" to "displayName asc"
$groupId = 'XXXXXXXX-22a5-4b09-839e-cf8960f697e4'
Get-MgGroupMember -GroupId $groupId -Property "id,displayName,userType,appId,mail,onPremisesSyncEnabled,deviceId" -Top 20 -Sort "displayName asc" -CountVariable CountVar -ConsistencyLevel eventual

#endregion

#region Invoke-MgGraphRequest supports different output types (-OutputType parameter)

$uri = 'https://graph.microsoft.com/beta/groups/XXXXXXXX-22a5-4b09-839e-cf8960f697e4/memberOf/$/microsoft.graph.group?$select=id,displayName,mailEnabled,securityEnabled,groupTypes,onPremisesSyncEnabled,mail,isAssignableToRole&$top=20&$orderby=displayName%20asc&$count=true'

Invoke-MgGraphRequest -Method GET $uri -OutputType PSObject -OutVariable outputPSObject

$outputPSObject.value

Invoke-MgGraphRequest -Method GET $uri -OutputType json -OutVariable outputJson

$outputJson | code -

Get-MgGroupMemberOf -GroupId XXXXXXXX-22a5-4b09-839e-cf8960f697e4 -OutVariable memberOf
$memberOf.ToJsonString() | ConvertFrom-Json 

# To retrieve the details of the signed-in user
# NOTE: Get-MgUser doesn't support the /me endpoint directly. Use -UserId $SignedInUser.Id
Invoke-MgGraphRequest -Method GET https://graph.microsoft.com/v1.0/me
Invoke-MgGraphRequest -Method GET /beta/me

Invoke-MgGraphRequest -Method GET /beta/me -OutputType HashTable -OutVariable outputHashtable # default
Invoke-MgGraphRequest -Method GET /beta/me -OutputType HttpResponseMessage -OutVariable outputResponse
Invoke-MgGraphRequest -Method GET /beta/me -OutputType Json -OutVariable outputJson
Invoke-MgGraphRequest -Method GET /beta/me -OutputType PSObject -OutVariable outputPSObject

$outputJson | code -

#endregion

#endregion